import { h } from "@stencil/core";
import classNames from "classnames";
import { ProviderTypeEnum } from "../../../types/provider.types";
import { EventBus } from "../../../utils/EventBus";
import { processImgSrc } from "../../../utils/processImgSrc";
import { getIsExtensionAvailable, getIsMetaMaskAvailable } from "./helpers";
import { UnlockPanelEventsEnum } from "./unlock-panel.types";
const unlockPanelClasses = {
    footerIcon: 'mvx:w-4! mvx:h-auto!',
};
export class UnlockPanel {
    constructor() {
        this.eventBus = new EventBus();
        this.isOpen = false;
        this.allowedProviders = [];
        this.isLoggingIn = false;
        this.isIntroScreenVisible = false;
        this.selectedMethod = null;
        this.hasSlotContent = false;
        this.panelState = {
            isOpen: false,
            allowedProviders: this.allowedProviders,
        };
        this.anchor = null;
        this.observer = null;
    }
    async getEventBus() {
        return this.eventBus;
    }
    handleIsOpenChange(newValue) {
        this.panelState = Object.assign(Object.assign({}, this.panelState), { isOpen: newValue });
    }
    handleAllowedProvidersChange(newValue) {
        this.panelState = Object.assign(Object.assign({}, this.panelState), { allowedProviders: newValue });
    }
    isExtensionInstalled(currentProvider) {
        return currentProvider === ProviderTypeEnum.extension && getIsExtensionAvailable();
    }
    isMetaMaskInstalled(currentProvider) {
        return currentProvider === ProviderTypeEnum.metamask && getIsMetaMaskAvailable();
    }
    async disconnectedCallback() {
        if (this.observer) {
            this.observer.disconnect();
            this.observer = null;
        }
        this.eventBus.unsubscribe(UnlockPanelEventsEnum.OPEN, this.unlockPanelUpdate.bind(this));
        this.eventBus.unsubscribe(UnlockPanelEventsEnum.CANCEL_IN_PROVIDER, this.handleResetLoginState.bind(this));
        this.isLoggingIn = false;
        this.selectedMethod = null;
        this.panelState = { isOpen: false, allowedProviders: [] };
        const delayClosingAnimation = new Promise(resolve => setTimeout(resolve, 300));
        return delayClosingAnimation;
    }
    componentDidLoad() {
        this.eventBus.subscribe(UnlockPanelEventsEnum.OPEN, this.unlockPanelUpdate.bind(this));
        this.eventBus.subscribe(UnlockPanelEventsEnum.CANCEL_IN_PROVIDER, this.handleResetLoginState.bind(this));
    }
    unlockPanelUpdate(payload) {
        this.panelState = Object.assign(Object.assign({}, payload), { allowedProviders: payload.allowedProviders });
    }
    handleLogin(provider) {
        this.selectedMethod = provider;
        switch (provider.type) {
            case ProviderTypeEnum.ledger:
                this.isIntroScreenVisible = true;
                break;
            case ProviderTypeEnum.crossWindow:
            case ProviderTypeEnum.extension:
            case ProviderTypeEnum.metamask:
            case ProviderTypeEnum.passkey:
                this.isIntroScreenVisible = true;
                this.isLoggingIn = true;
                this.eventBus.publish(UnlockPanelEventsEnum.LOGIN, { type: provider.type, anchor: this.anchor });
                break;
            default:
                this.handleAccess();
        }
    }
    handleResetLoginState(event) {
        var _a;
        (_a = event === null || event === void 0 ? void 0 : event.preventDefault) === null || _a === void 0 ? void 0 : _a.call(event);
        this.isLoggingIn = false;
        this.isIntroScreenVisible = false;
        this.selectedMethod = null;
        if (!this.anchor) {
            return;
        }
        while (this.anchor.firstChild) {
            this.anchor.removeChild(this.anchor.firstChild);
        }
        this.eventBus.publish(UnlockPanelEventsEnum.CANCEL_LOGIN);
    }
    handleClose(event) {
        event.preventDefault();
        if (this.selectedMethod) {
            this.eventBus.publish(UnlockPanelEventsEnum.CANCEL_LOGIN);
        }
        this.eventBus.publish(UnlockPanelEventsEnum.CLOSE);
    }
    handleAccess() {
        this.isIntroScreenVisible = false;
        this.isLoggingIn = true;
        this.eventBus.publish(UnlockPanelEventsEnum.LOGIN, { type: this.selectedMethod.type, anchor: this.anchor });
    }
    render() {
        const detectedProviders = this.panelState.allowedProviders.filter(allowedProvider => this.isExtensionInstalled(allowedProvider.type) || this.isMetaMaskInstalled(allowedProvider.type));
        const otherProviders = this.panelState.allowedProviders.filter(allowedProvider => !detectedProviders.includes(allowedProvider));
        const panelTitle = this.selectedMethod ? this.selectedMethod.name : 'Connect your wallet';
        const hasDetectedProviders = detectedProviders.length > 0;
        const isProviderScreenVisible = !this.isLoggingIn && !this.isIntroScreenVisible;
        return (h("mvx-side-panel", { key: '1704290555049dd046e406bcea35f3722ca7bf3b', isOpen: this.panelState.isOpen, panelTitle: panelTitle, withBackButton: this.isLoggingIn, onClose: this.handleClose.bind(this), onBack: this.handleResetLoginState.bind(this) }, h("div", { key: '2c8abacf4c57f00cf836eb4bf1567942dd6d6991', id: "anchor", ref: el => {
                this.anchor = el;
            }, class: { 'unlock-panel-anchor': this.isLoggingIn } }, this.isIntroScreenVisible && h("mvx-unlock-provider-intro", { key: 'f7b4bbd421821be639656ebe4f1b11bff2bb6a65', provider: this.selectedMethod, onAccess: this.handleAccess.bind(this) })), isProviderScreenVisible && (h("div", { key: '97c7be035f35adeaf1c5351c9dd68ff7b9250549', class: "unlock-panel" }, h("div", { key: 'bef1d82cf10db58f9dcef97c183f47c9965b9d1d', class: "unlock-panel-groups" }, hasDetectedProviders && (h("div", { key: 'dc1179f8a63f42bea027d529994ee8bd8bf62912', class: "unlock-panel-group" }, h("div", { key: 'f1b912449d59e6e23b3dc8165538b90b9eb0ecc8', class: "unlock-panel-group-label" }, "Detected"), h("div", { key: '4e5593a10da2510e98466d1623252e77eddeb35a', class: "unlock-panel-group-providers" }, detectedProviders.map(provider => (h("mvx-unlock-provider-button", { provider: provider, onClick: this.handleLogin.bind(this, provider) })))))), h("div", { key: '3b6c0710971fcb2f1097856c47fea79ca12cfabd', class: "unlock-panel-group" }, h("div", { key: '0c525495d3c0d28f748dbab20296e875e0545cd2', class: "unlock-panel-group-label" }, hasDetectedProviders ? 'Other Options' : 'Options'), h("div", { key: '59e0bea07f87474cd6e27466027f64a559ab8941', class: "unlock-panel-group-providers" }, otherProviders.map(provider => (h("mvx-unlock-provider-button", { provider: provider, onClick: this.handleLogin.bind(this, provider) }))), h("slot", { key: '918e5b7d0eac9f046490f0b90009a20088f8e0d4' })))), h("div", { key: 'a079cb7b185f29925c6c7fb137b18420075f2b0f', class: "unlock-panel-footer" }, h("img", { key: '6d13d2e4552854cad6e2aa663efcf829489317a8', src: processImgSrc('unlock-panel-wallet.png'), class: "unlock-panel-footer-image" }), h("div", { key: 'f9ab9750281554b0b90a9a9613473b8291ae1d3e', class: "unlock-panel-footer-wrapper" }, h("div", { key: '40526398c5518d6056b19216b015ba9ff94366cd', class: "unlock-panel-footer-title" }, "Don't have a wallet?"), h("div", { key: '2d6378cd9a4b1947c6de5e66c45823e336db66c5', class: "unlock-panel-footer-subtitle" }, "Take full control of ", h("br", { key: 'e9ee879c3a31b1cea9ebc8978e93b54f47545c0d' }), " your assets."), h("mvx-arrow-up-right-icon", { key: '2b3096d5976cd9a41fe49b2c1250721734601b34', class: classNames('unlock-panel-footer-icon', {
                [unlockPanelClasses.footerIcon]: true,
            }) })))))));
    }
    static get is() { return "mvx-unlock-panel"; }
    static get encapsulation() { return "shadow"; }
    static get originalStyleUrls() {
        return {
            "$": ["unlock-panel.scss"]
        };
    }
    static get styleUrls() {
        return {
            "$": ["unlock-panel.css"]
        };
    }
    static get properties() {
        return {
            "isOpen": {
                "type": "boolean",
                "attribute": "is-open",
                "mutable": false,
                "complexType": {
                    "original": "boolean",
                    "resolved": "boolean",
                    "references": {}
                },
                "required": false,
                "optional": false,
                "docs": {
                    "tags": [],
                    "text": ""
                },
                "getter": false,
                "setter": false,
                "reflect": false,
                "defaultValue": "false"
            },
            "allowedProviders": {
                "type": "unknown",
                "attribute": "allowed-providers",
                "mutable": false,
                "complexType": {
                    "original": "IProviderBase[]",
                    "resolved": "IProviderBase<ProviderTypeEnum>[]",
                    "references": {
                        "IProviderBase": {
                            "location": "import",
                            "path": "types/provider.types",
                            "id": "src/types/provider.types.ts::IProviderBase"
                        }
                    }
                },
                "required": false,
                "optional": false,
                "docs": {
                    "tags": [],
                    "text": ""
                },
                "getter": false,
                "setter": false,
                "defaultValue": "[]"
            }
        };
    }
    static get states() {
        return {
            "isLoggingIn": {},
            "isIntroScreenVisible": {},
            "selectedMethod": {},
            "hasSlotContent": {},
            "panelState": {}
        };
    }
    static get methods() {
        return {
            "getEventBus": {
                "complexType": {
                    "signature": "() => Promise<IEventBus>",
                    "parameters": [],
                    "references": {
                        "Promise": {
                            "location": "global",
                            "id": "global::Promise"
                        },
                        "IEventBus": {
                            "location": "import",
                            "path": "utils/EventBus",
                            "id": "src/utils/EventBus.ts::IEventBus"
                        }
                    },
                    "return": "Promise<IEventBus>"
                },
                "docs": {
                    "text": "",
                    "tags": []
                }
            }
        };
    }
    static get elementRef() { return "hostElement"; }
    static get watchers() {
        return [{
                "propName": "isOpen",
                "methodName": "handleIsOpenChange"
            }, {
                "propName": "allowedProviders",
                "methodName": "handleAllowedProvidersChange"
            }];
    }
}
//# sourceMappingURL=unlock-panel.js.map
